
#include "databasewrapper.h"
#include "sqlite3/sqlite3.h"
#include "id3.h"
#include "id3/misc_support.h"
#include <io.h>
#include "debug.h"

/*      http://torjo.blogspot.com/2008/03/directories-flawed-concept-example-of.html
Say I have 4 categories: HR, Document, Resume, Report.

    * I have the following files:
          o My Test Doc, belonging to Document
          o Johns Report, belonging to Document, Report
          o Yearly Report, belonging to Document, Report
          o My Recruiting Report, belonging to HR, Document, Report
          o Victors Resume, belonging to HR, Document, Resume
          o HR Evaluation, belonging to HR, Document, Report

Browsing away:
    * Im in /Everything. I see all 6 files and 4 categories
          o I expand to Document
                + Im in /Everything/Document
                + I see HR, Resume, Report categories
                + I see all 6 files
          o I expand to HR
                + Im in /Everything/Document/HR
                + I see Resume, Report categories
                + I see 3 files: My Recruiting Report, Victors Resume, HR evaluation
          o I expand to Report
                + Im in /Everything/Document/HR/Report
                + I dont see any more categories
                + I see 2 files: My Recruiting Report, HR evaluation

Browsing away, take 2:
    * Im in /Everything. I see 6 files and 4 categories
          o I expand to Report
                + Im in /Everything/Report
                + I see Document, HR categories
                + Note: here I dont see Resume category, since there arent any files that belong to both Report and Resume
                + I see 4 files: Johns report, Yearly Report, My Recruiting Report, HR Evaluation
          o I expand to HR
                + Im in /Everything/Report/HR
                + I see Document category
                + I see 2 files: My Recruiting Report, HR Evaluation
          o I expand to Document
                + Im in /Everything/Report/HR/Document
                + I dont see any more categories
                + I see 2 files: My Recruiting Report, HR Evaluation 

This is just one way to explore categories. There can be other ways - I'm all for diversity...
*/

const int ActVersion = 0x0100;

sqlite3 *db;
sqlite3_stmt *query;

DatabaseWrapper::DatabaseWrapper() {
   log("->  DatabaseWrapper::DatabaseWrapper()\n");
   sqlite3_open("files.db", &db);

   bool needToCreateTables = true;

   char queryStr[] = "SELECT version FROM MetaFSConfig";
   sqlite3_prepare(db, queryStr, strlen(queryStr), &query, NULL);

   while(sqlite3_step(query) == SQLITE_ROW) {
      unsigned int versionInDB = sqlite3_column_int(query, 0);
      if(versionInDB == ActVersion)
         needToCreateTables = false;
   }
   sqlite3_finalize(query);

   if(needToCreateTables) {
      sqlite3_close(db);
      unlink("files.db");
      sqlite3_open("files.db", &db);
      createTablesAndInitFromMP3();
   }

   log("<-  DatabaseWrapper::DatabaseWrapper()\n");
}

DatabaseWrapper::~DatabaseWrapper() {
   log("->  DatabaseWrapper::~DatabaseWrapper()\n");
   sqlite3_close(db);
   log("<-  DatabaseWrapper::~DatabaseWrapper()\n");
}

void DatabaseWrapper::createTablesAndInitFromMP3() {
   log("->  DatabaseWrapper::createTables()\n");
   char command[512];
   char *errmsg = 0;
   sqlite3_exec(db, "CREATE TABLE MetaFSConfig (version INTEGER PRIMARY KEY)", NULL, NULL, &errmsg);
   sprintf(command, "INSERT INTO MetaFSConfig (version) VALUES('%i');", ActVersion);
   sqlite3_exec(db, command, NULL, NULL, &errmsg);

   sqlite3_exec(db, "CREATE TABLE files (key INTEGER PRIMARY KEY,filename TEXT,path TEXT,tags TEXT)", NULL, NULL, &errmsg);
   scanOsDirectory("c:\\test\\", 0);

   //now scan all files and get id3-tags
   sqlite3_stmt *fileQuery;
   char query[] = "SELECT * FROM files;";
   sqlite3_prepare(db, query, strlen(query), &fileQuery, NULL);
   while(sqlite3_step(fileQuery) == SQLITE_ROW) {
      const unsigned char *hold;
      char filename[512];
      char path[512];
      int id;

      id = sqlite3_column_int(fileQuery, 0);    //id

      hold = sqlite3_column_text(fileQuery, 1);   //filename
      if(hold)
         strncpy(filename, (const char*)hold, 512);

      hold = sqlite3_column_text(fileQuery, 2);   //path
      if(hold)
         strncpy(path, (const char*)hold, 40);

      std::string completeFilename = path;
      completeFilename += filename;

      ID3_Tag tag(completeFilename.c_str());
      //Title
      {
         char *title = ID3_GetTitle(&tag);
         if(title != NULL) {
            insertValueAndFileIntoTag("title", id, title);
            delete[] title;
         }
      }

      //Album
      {
         char *album = ID3_GetAlbum(&tag);
         if(album != NULL) {
            insertValueAndFileIntoTag("album", id, album);
            delete[] album;
         }
      }

      //Artist
      {
         char *artist = ID3_GetArtist(&tag);
         if(artist != NULL) {
            insertValueAndFileIntoTag("artist", id, artist);
            delete[] artist;
         }
      }

      //Genre
      {
         size_t genreNum = ID3_GetGenreNum(&tag);
         const char *genre = ID3_V1GENRE2DESCRIPTION(genreNum);
         if(genre != NULL) {
            insertValueAndFileIntoTag("genre", id, genre);
         }
      }
   }
   sqlite3_finalize(fileQuery);
   log("<-  DatabaseWrapper::createTables()\n");
}

//----------------------------------------------------------------------------------
// insertIntoTag
//----------------------------------------------------------------------------------
// 
void DatabaseWrapper::insertValueAndFileIntoTag(const char *tagName,const int fileId,const char *value) {
   char *errmsg = 0;
   char command[512];

   sqlite3_stmt *tagQuery;
   char query[] = "SELECT * FROM tag_%s;";
   sprintf(command, "SELECT * FROM tag_%s;", tagName);
   sqlite3_prepare(db, query, strlen(query), &tagQuery, NULL);
   if(sqlite3_step(tagQuery) != SQLITE_ROW) {      //already available?
      createTag(tagName);
   } 
   sqlite3_finalize(tagQuery);

   sprintf(command, "INSERT INTO tag_%s (fileId,value) VALUES (%i,'%s');",tagName, fileId, value);
   sqlite3_exec(db, command, NULL, NULL, &errmsg);

   //insert tag into text-entry from file
   sprintf(command, "SELECT tags FROM files WHERE key=%i;", fileId);
   sqlite3_prepare(db, command, strlen(command), &tagQuery, NULL);
   sqlite3_step(tagQuery);
   const char *tags = (const char*)sqlite3_column_text(tagQuery, 0);
   char newTags[1024];
   sprintf(newTags, "%s,%s", tags, tagName);
   sqlite3_finalize(tagQuery);

   sprintf(command, "UPDATE files SET tags='%s' WHERE key=%i;", newTags, fileId);
   sqlite3_exec(db, command, NULL, NULL, &errmsg);

}

//----------------------------------------------------------------------------------
// createTag
//----------------------------------------------------------------------------------
// 
void DatabaseWrapper::createTag(const char *tagName) {
   char *errmsg = 0;
   char command[512];
   sprintf(command, "CREATE TABLE tag_%s (key INTEGER PRIMARY KEY, fileId INTEGER, value TEXT);", tagName);
   sqlite3_exec(db, command, NULL, NULL, &errmsg);
}

//----------------------------------------------------------------------------------
// addDirectory
//----------------------------------------------------------------------------------
// 
void DatabaseWrapper::scanOsDirectory(std::string strDirName, int level) {
   log("->  DatabaseWrapper::addDirectory()\n");
   _finddata_t wfd;
   long hFind;
   std::string strText;

   strText = strDirName + "*";

   hFind = _findfirst(strText.c_str(), &wfd);
   if(hFind != -1L) {
      do {
         if ((strcmp(wfd.name,".")) && (strcmp(wfd.name,"..")) && (wfd.attrib & _A_SUBDIR)) {

            strText = strDirName + wfd.name;

            // Recurse Dirs
            if(level==0) {
               if(strText.find("RECYCLER",0) == -1)	//remove recycler from tree
                  scanOsDirectory(strText+"\\", level+1);
            } else {
               scanOsDirectory(strText+"\\", level+1);
            }
         }
         if((wfd.attrib & _A_SUBDIR) == 0) {    //TODO: auf .mp3 abfragen!
            strText = strDirName + wfd.name;
            sprintf(mSqlCommand, "INSERT INTO files (filename, path) VALUES('%s','%s');", wfd.name, strDirName.c_str());
            char *errmsg = 0;
            sqlite3_exec(db, mSqlCommand, NULL, NULL, &errmsg);
         }
      } while (_findnext(hFind, &wfd) == 0);
      _findclose(hFind);
   }
   log("<-  DatabaseWrapper::addDirectory()\n");
}
